@@ -61,11 +61,10 @@ i965_avc_bsd_free_avc_bsd_surface(void **data)
 static void
 i965_avc_bsd_init_avc_bsd_surface(VADriverContextP ctx, 
                                   struct object_surface *obj_surface,
-                                  VAPictureParameterBufferH264 *pic_param)
+                                  VAPictureParameterBufferH264 *pic_param,
+                                  struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_bsd_context *i965_avc_bsd_context = &i965_h264_context->i965_avc_bsd_context;
     struct i965_avc_bsd_surface *avc_bsd_surface = obj_surface->private_data;
 
@@ -98,25 +97,31 @@ i965_avc_bsd_init_avc_bsd_surface(VADriverContextP ctx,
 }
 
 static void
-i965_bsd_ind_obj_base_address(VADriverContextP ctx, struct decode_state *decode_state, int slice)
+i965_bsd_ind_obj_base_address(VADriverContextP ctx,
+                              struct decode_state *decode_state,
+                              int slice,
+                              struct i965_h264_context *i965_h264_context)
+                              
 {
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
+
     dri_bo *ind_bo = decode_state->slice_datas[slice]->bo;
 
-    BEGIN_BCS_BATCH(ctx, 3);
-    OUT_BCS_BATCH(ctx, CMD_BSD_IND_OBJ_BASE_ADDR | (3 - 2));
-    OUT_BCS_RELOC(ctx, ind_bo,
+    BEGIN_BCS_BATCH(batch, 3);
+    OUT_BCS_BATCH(batch, CMD_BSD_IND_OBJ_BASE_ADDR | (3 - 2));
+    OUT_BCS_RELOC(batch, ind_bo,
                   I915_GEM_DOMAIN_INSTRUCTION, 0,
                   0);
-    OUT_BCS_BATCH(ctx, 0);
-    ADVANCE_BCS_BATCH(ctx);
+    OUT_BCS_BATCH(batch, 0);
+    ADVANCE_BCS_BATCH(batch);
 }
 
 static void
-i965_avc_bsd_img_state(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_bsd_img_state(VADriverContextP ctx,
+                       struct decode_state *decode_state,
+                       struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     int qm_present_flag;
     int img_struct;
     int mbaff_frame_flag;
@@ -169,14 +174,14 @@ i965_avc_bsd_img_state(VADriverContextP ctx, struct decode_state *decode_state)
 
     avc_it_command_header = (CMD_MEDIA_OBJECT_EX | (12 - 2));
 
-    BEGIN_BCS_BATCH(ctx, 6);
-    OUT_BCS_BATCH(ctx, CMD_AVC_BSD_IMG_STATE | (6 - 2));
-    OUT_BCS_BATCH(ctx, 
+    BEGIN_BCS_BATCH(batch, 6);
+    OUT_BCS_BATCH(batch, CMD_AVC_BSD_IMG_STATE | (6 - 2));
+    OUT_BCS_BATCH(batch, 
                   ((width_in_mbs * height_in_mbs) & 0x7fff));
-    OUT_BCS_BATCH(ctx, 
+    OUT_BCS_BATCH(batch, 
                   (height_in_mbs << 16) | 
                   (width_in_mbs << 0));
-    OUT_BCS_BATCH(ctx, 
+    OUT_BCS_BATCH(batch, 
                   ((pic_param->second_chroma_qp_index_offset & 0x1f) << 24) |
                   ((pic_param->chroma_qp_index_offset & 0x1f) << 16) | 
                   (SCAN_RASTER_ORDER << 15) | /* AVC ILDB Data */
@@ -186,7 +191,7 @@ i965_avc_bsd_img_state(VADriverContextP ctx, struct decode_state *decode_state)
                   (qm_present_flag << 10) |
                   (img_struct << 8) |
                   (16 << 0)); /* FIXME: always support 16 reference frames ??? */
-    OUT_BCS_BATCH(ctx,
+    OUT_BCS_BATCH(batch,
                   (RESIDUAL_DATA_OFFSET << 24) | /* residual data offset */
                   (0 << 17) | /* don't overwrite SRT */
                   (0 << 16) | /* Un-SRT (Unsynchronized Root Thread) */
@@ -201,13 +206,16 @@ i965_avc_bsd_img_state(VADriverContextP ctx, struct decode_state *decode_state)
                   (pic_param->seq_fields.bits.frame_mbs_only_flag << 2) |
                   (mbaff_frame_flag << 1) |
                   (pic_param->pic_fields.bits.field_pic_flag << 0));
-    OUT_BCS_BATCH(ctx, avc_it_command_header);
-    ADVANCE_BCS_BATCH(ctx);
+    OUT_BCS_BATCH(batch, avc_it_command_header);
+    ADVANCE_BCS_BATCH(batch);
 }
 
 static void
-i965_avc_bsd_qm_state(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_bsd_qm_state(VADriverContextP ctx,
+                      struct decode_state *decode_state,
+                      struct i965_h264_context *i965_h264_context)
 {
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     int cmd_len;
     VAIQMatrixBufferH264 *iq_matrix;
     VAPictureParameterBufferH264 *pic_param;
@@ -225,34 +233,33 @@ i965_avc_bsd_qm_state(VADriverContextP ctx, struct decode_state *decode_state)
     if (pic_param->pic_fields.bits.transform_8x8_mode_flag)
         cmd_len += 2 * 16; /* load two 8x8 scaling matrices */
 
-    BEGIN_BCS_BATCH(ctx, cmd_len);
-    OUT_BCS_BATCH(ctx, CMD_AVC_BSD_QM_STATE | (cmd_len - 2));
+    BEGIN_BCS_BATCH(batch, cmd_len);
+    OUT_BCS_BATCH(batch, CMD_AVC_BSD_QM_STATE | (cmd_len - 2));
 
     if (pic_param->pic_fields.bits.transform_8x8_mode_flag)
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (0x0  << 8) | /* don't use default built-in matrices */
                       (0xff << 0)); /* six 4x4 and two 8x8 scaling matrices */
     else
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (0x0  << 8) | /* don't use default built-in matrices */
                       (0x3f << 0)); /* six 4x4 scaling matrices */
 
-    intel_batchbuffer_data_bcs(ctx, &iq_matrix->ScalingList4x4[0][0], 6 * 4 * 4);
+    intel_batchbuffer_data(batch, &iq_matrix->ScalingList4x4[0][0], 6 * 4 * 4);
 
     if (pic_param->pic_fields.bits.transform_8x8_mode_flag)
-        intel_batchbuffer_data_bcs(ctx, &iq_matrix->ScalingList8x8[0][0], 2 * 16 * 4);
+        intel_batchbuffer_data(batch, &iq_matrix->ScalingList8x8[0][0], 2 * 16 * 4);
 
-    ADVANCE_BCS_BATCH(ctx);
+    ADVANCE_BCS_BATCH(batch);
 }
 
 static void
 i965_avc_bsd_slice_state(VADriverContextP ctx, 
                          VAPictureParameterBufferH264 *pic_param, 
-                         VASliceParameterBufferH264 *slice_param)
+                         VASliceParameterBufferH264 *slice_param,
+                         struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     int present_flag, cmd_len, list, j;
     struct {
         unsigned char bottom_idc:1;
@@ -292,9 +299,9 @@ i965_avc_bsd_slice_state(VADriverContextP ctx,
         cmd_len += 96;
     }
 
-    BEGIN_BCS_BATCH(ctx, cmd_len);
-    OUT_BCS_BATCH(ctx, CMD_AVC_BSD_SLICE_STATE | (cmd_len - 2));
-    OUT_BCS_BATCH(ctx, present_flag);
+    BEGIN_BCS_BATCH(batch, cmd_len);
+    OUT_BCS_BATCH(batch, CMD_AVC_BSD_SLICE_STATE | (cmd_len - 2));
+    OUT_BCS_BATCH(batch, present_flag);
 
     for (list = 0; list < 2; list++) {
         int flag;
@@ -343,7 +350,7 @@ i965_avc_bsd_slice_state(VADriverContextP ctx,
             va_pic++;
         }
         
-        intel_batchbuffer_data_bcs(ctx, refs, sizeof(refs));
+        intel_batchbuffer_data(batch, refs, sizeof(refs));
     }
 
     i965_h264_context->weight128_luma_l0 = 0;
@@ -382,7 +389,7 @@ i965_avc_bsd_slice_state(VADriverContextP ctx,
             }
         }
 
-        intel_batchbuffer_data_bcs(ctx, weightoffsets, sizeof(weightoffsets));
+        intel_batchbuffer_data(batch, weightoffsets, sizeof(weightoffsets));
     }
 
     if (present_flag & PRESENT_WEIGHT_OFFSET_L1) {
@@ -411,51 +418,49 @@ i965_avc_bsd_slice_state(VADriverContextP ctx,
             }
         }
 
-        intel_batchbuffer_data_bcs(ctx, weightoffsets, sizeof(weightoffsets));
+        intel_batchbuffer_data(batch, weightoffsets, sizeof(weightoffsets));
     }
 
-    ADVANCE_BCS_BATCH(ctx);
+    ADVANCE_BCS_BATCH(batch);
 }
 
 static void
 i965_avc_bsd_buf_base_state(VADriverContextP ctx,
                             VAPictureParameterBufferH264 *pic_param, 
-                            VASliceParameterBufferH264 *slice_param)
+                            VASliceParameterBufferH264 *slice_param,
+                            struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     struct i965_avc_bsd_context *i965_avc_bsd_context;
     int i, j;
     VAPictureH264 *va_pic;
     struct object_surface *obj_surface;
     struct i965_avc_bsd_surface *avc_bsd_surface;
 
-    assert(media_state->private_context);
-    i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     i965_avc_bsd_context = &i965_h264_context->i965_avc_bsd_context;
 
-    BEGIN_BCS_BATCH(ctx, 74);
-    OUT_BCS_BATCH(ctx, CMD_AVC_BSD_BUF_BASE_STATE | (74 - 2));
-    OUT_BCS_RELOC(ctx, i965_avc_bsd_context->bsd_raw_store.bo,
+    BEGIN_BCS_BATCH(batch, 74);
+    OUT_BCS_BATCH(batch, CMD_AVC_BSD_BUF_BASE_STATE | (74 - 2));
+    OUT_BCS_RELOC(batch, i965_avc_bsd_context->bsd_raw_store.bo,
                   I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                   0);
-    OUT_BCS_RELOC(ctx, i965_avc_bsd_context->mpr_row_store.bo,
+    OUT_BCS_RELOC(batch, i965_avc_bsd_context->mpr_row_store.bo,
                   I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                   0);
-    OUT_BCS_RELOC(ctx, i965_h264_context->avc_it_command_mb_info.bo,
+    OUT_BCS_RELOC(batch, i965_h264_context->avc_it_command_mb_info.bo,
                   I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                   i965_h264_context->avc_it_command_mb_info.mbs * i965_h264_context->use_avc_hw_scoreboard * MB_CMD_IN_BYTES);
-    OUT_BCS_RELOC(ctx, i965_h264_context->avc_it_data.bo,
+    OUT_BCS_RELOC(batch, i965_h264_context->avc_it_data.bo,
                   I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                   (i965_h264_context->avc_it_data.write_offset << 6));
 
     if (i965_h264_context->enable_avc_ildb)
-        OUT_BCS_RELOC(ctx, i965_h264_context->avc_ildb_data.bo,
+        OUT_BCS_RELOC(batch, i965_h264_context->avc_ildb_data.bo,
                       I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                       0);
     else
-        OUT_BCS_BATCH(ctx, 0);
+        OUT_BCS_BATCH(batch, 0);
 
     for (i = 0; i < ARRAY_ELEMS(i965_h264_context->fsid_list); i++) {
         if (i965_h264_context->fsid_list[i].surface_id != VA_INVALID_ID) {
@@ -480,26 +485,26 @@ i965_avc_bsd_buf_base_state(VADriverContextP ctx,
                 avc_bsd_surface = obj_surface->private_data;
             
                 if (avc_bsd_surface == NULL) {
-                    OUT_BCS_BATCH(ctx, 0);
-                    OUT_BCS_BATCH(ctx, 0);
+                    OUT_BCS_BATCH(batch, 0);
+                    OUT_BCS_BATCH(batch, 0);
                 } else {
-                    OUT_BCS_RELOC(ctx, avc_bsd_surface->dmv_top,
+                    OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
                                   I915_GEM_DOMAIN_INSTRUCTION, 0,
                                   0);
 
                     if (avc_bsd_surface->dmv_bottom_flag == 1)
-                        OUT_BCS_RELOC(ctx, avc_bsd_surface->dmv_bottom,
+                        OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_bottom,
                                       I915_GEM_DOMAIN_INSTRUCTION, 0,
                                       0);
                     else
-                        OUT_BCS_RELOC(ctx, avc_bsd_surface->dmv_top,
+                        OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
                                       I915_GEM_DOMAIN_INSTRUCTION, 0,
                                       0);
                 }
             } 
         } else {
-            OUT_BCS_BATCH(ctx, 0);
-            OUT_BCS_BATCH(ctx, 0);
+            OUT_BCS_BATCH(batch, 0);
+            OUT_BCS_BATCH(batch, 0);
         }
     }
 
@@ -509,26 +514,20 @@ i965_avc_bsd_buf_base_state(VADriverContextP ctx,
     assert(obj_surface);
     obj_surface->flags &= ~SURFACE_REF_DIS_MASK;
     obj_surface->flags |= (pic_param->pic_fields.bits.reference_pic_flag ? SURFACE_REFERENCED : 0);
-    i965_avc_bsd_init_avc_bsd_surface(ctx, obj_surface, pic_param);
+    i965_avc_bsd_init_avc_bsd_surface(ctx, obj_surface, pic_param, i965_h264_context);
     avc_bsd_surface = obj_surface->private_data;
+    i965_check_alloc_surface_bo(ctx, obj_surface, 0, VA_FOURCC('N','V','1','2'));
 
-    if (obj_surface->bo == NULL) {
-        obj_surface->bo = dri_bo_alloc(i965->intel.bufmgr,
-                                       "vaapi surface",
-                                       obj_surface->size,
-                                       0x1000);
-    }
-
-    OUT_BCS_RELOC(ctx, avc_bsd_surface->dmv_top,
+    OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
                   I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                   0);
 
     if (avc_bsd_surface->dmv_bottom_flag == 1)
-        OUT_BCS_RELOC(ctx, avc_bsd_surface->dmv_bottom,
+        OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_bottom,
                       I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                       0);
     else
-        OUT_BCS_RELOC(ctx, avc_bsd_surface->dmv_top,
+        OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
                       I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
                       0);
 
@@ -551,20 +550,20 @@ i965_avc_bsd_buf_base_state(VADriverContextP ctx,
             assert(found == 1);
 
             if (!(va_pic->flags & VA_PICTURE_H264_INVALID)) {
-                OUT_BCS_BATCH(ctx, va_pic->TopFieldOrderCnt);
-                OUT_BCS_BATCH(ctx, va_pic->BottomFieldOrderCnt);
+                OUT_BCS_BATCH(batch, va_pic->TopFieldOrderCnt);
+                OUT_BCS_BATCH(batch, va_pic->BottomFieldOrderCnt);
             } 
         } else {
-            OUT_BCS_BATCH(ctx, 0);
-            OUT_BCS_BATCH(ctx, 0);
+            OUT_BCS_BATCH(batch, 0);
+            OUT_BCS_BATCH(batch, 0);
         }
     }
 
     va_pic = &pic_param->CurrPic;
-    OUT_BCS_BATCH(ctx, va_pic->TopFieldOrderCnt);
-    OUT_BCS_BATCH(ctx, va_pic->BottomFieldOrderCnt);
+    OUT_BCS_BATCH(batch, va_pic->TopFieldOrderCnt);
+    OUT_BCS_BATCH(batch, va_pic->BottomFieldOrderCnt);
 
-    ADVANCE_BCS_BATCH(ctx);
+    ADVANCE_BCS_BATCH(batch);
 }
 
 /*
@@ -601,8 +600,10 @@ g4x_avc_bsd_object(VADriverContextP ctx,
                    struct decode_state *decode_state,
                    VAPictureParameterBufferH264 *pic_param,
                    VASliceParameterBufferH264 *slice_param,
-                   int slice_index)
+                   int slice_index,
+                   struct i965_h264_context *i965_h264_context)
 {
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     int width_in_mbs = pic_param->picture_width_in_mbs_minus1 + 1;
     int height_in_mbs = pic_param->picture_height_in_mbs_minus1 + 1; /* frame height */
 
@@ -667,15 +668,15 @@ g4x_avc_bsd_object(VADriverContextP ctx,
         slice_hor_pos = first_mb_in_slice % width_in_mbs; 
         slice_ver_pos = first_mb_in_slice / width_in_mbs;
 
-        BEGIN_BCS_BATCH(ctx, cmd_len);
-        OUT_BCS_BATCH(ctx, CMD_AVC_BSD_OBJECT | (cmd_len - 2));
-        OUT_BCS_BATCH(ctx, 
+        BEGIN_BCS_BATCH(batch, cmd_len);
+        OUT_BCS_BATCH(batch, CMD_AVC_BSD_OBJECT | (cmd_len - 2));
+        OUT_BCS_BATCH(batch, 
                       (encrypted << 31) |
                       ((slice_param->slice_data_size - (slice_data_bit_offset >> 3)) << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (slice_param->slice_data_offset +
                        (slice_data_bit_offset >> 3)));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (0 << 31) | /* concealment mode: 0->intra 16x16 prediction, 1->inter P Copy */
                       (0 << 14) | /* ignore BSDPrematureComplete Error handling */
                       (0 << 13) | /* FIXME: ??? */
@@ -683,12 +684,12 @@ g4x_avc_bsd_object(VADriverContextP ctx,
                       (0 << 10) | /* ignore Entropy Error handling */
                       (0 << 8)  | /* ignore MB Header Error handling */
                       (slice_type << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (num_ref_idx_l1 << 24) |
                       (num_ref_idx_l0 << 16) |
                       (slice_param->chroma_log2_weight_denom << 8) |
                       (slice_param->luma_log2_weight_denom << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (weighted_pred_idc << 30) |
                       (slice_param->direct_spatial_mv_pred_flag << 29) |
                       (slice_param->disable_deblocking_filter_idc << 27) |
@@ -696,30 +697,30 @@ g4x_avc_bsd_object(VADriverContextP ctx,
                       ((pic_param->pic_init_qp_minus26 + 26 + slice_param->slice_qp_delta) << 16) |
                       ((slice_param->slice_beta_offset_div2 & 0xf) << 8) |
                       ((slice_param->slice_alpha_c0_offset_div2 & 0xf) << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (slice_ver_pos << 24) |
                       (slice_hor_pos << 16) | 
                       (first_mb_in_slice << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (0 << 7) | /* FIXME: ??? */
                       ((0x7 - (slice_data_bit_offset & 0x7)) << 0));
 
         if (encrypted) {
-            OUT_BCS_BATCH(ctx, counter_value);
+            OUT_BCS_BATCH(batch, counter_value);
         }
 
-        ADVANCE_BCS_BATCH(ctx); 
+        ADVANCE_BCS_BATCH(batch); 
     } else {
-        BEGIN_BCS_BATCH(ctx, 8); 
-        OUT_BCS_BATCH(ctx, CMD_AVC_BSD_OBJECT | (8 - 2));
-        OUT_BCS_BATCH(ctx, 0); /* indirect data length for phantom slice is 0 */
-        OUT_BCS_BATCH(ctx, 0); /* indirect data start address for phantom slice is 0 */
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, width_in_mbs * height_in_mbs / (1 + !!pic_param->pic_fields.bits.field_pic_flag));
-        OUT_BCS_BATCH(ctx, 0);
-        ADVANCE_BCS_BATCH(ctx);
+        BEGIN_BCS_BATCH(batch, 8); 
+        OUT_BCS_BATCH(batch, CMD_AVC_BSD_OBJECT | (8 - 2));
+        OUT_BCS_BATCH(batch, 0); /* indirect data length for phantom slice is 0 */
+        OUT_BCS_BATCH(batch, 0); /* indirect data start address for phantom slice is 0 */
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, width_in_mbs * height_in_mbs / (1 + !!pic_param->pic_fields.bits.field_pic_flag));
+        OUT_BCS_BATCH(batch, 0);
+        ADVANCE_BCS_BATCH(batch);
     }
 }
 
@@ -728,15 +729,14 @@ ironlake_avc_bsd_object(VADriverContextP ctx,
                         struct decode_state *decode_state,
                         VAPictureParameterBufferH264 *pic_param,
                         VASliceParameterBufferH264 *slice_param,
-                        int slice_index)
+                        int slice_index,
+                        struct i965_h264_context *i965_h264_context)
 {
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     int width_in_mbs = pic_param->picture_width_in_mbs_minus1 + 1;
     int height_in_mbs = pic_param->picture_height_in_mbs_minus1 + 1; /* frame height */
 
     if (slice_param) {
-        struct i965_driver_data *i965 = i965_driver_data(ctx);
-        struct i965_media_state *media_state = &i965->media_state;
-        struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
         int encrypted, counter_value;
         int slice_hor_pos, slice_ver_pos;
         int num_ref_idx_l0, num_ref_idx_l1;
@@ -796,17 +796,17 @@ ironlake_avc_bsd_object(VADriverContextP ctx,
         slice_hor_pos = first_mb_in_slice % width_in_mbs; 
         slice_ver_pos = first_mb_in_slice / width_in_mbs;
 
-        BEGIN_BCS_BATCH(ctx, 16);
-        OUT_BCS_BATCH(ctx, CMD_AVC_BSD_OBJECT | (16 - 2));
-        OUT_BCS_BATCH(ctx, 
+        BEGIN_BCS_BATCH(batch, 16);
+        OUT_BCS_BATCH(batch, CMD_AVC_BSD_OBJECT | (16 - 2));
+        OUT_BCS_BATCH(batch, 
                       (encrypted << 31) |
                       (0 << 30) | /* FIXME: packet based bit stream */
                       (0 << 29) | /* FIXME: packet format */
                       ((slice_param->slice_data_size - (slice_data_bit_offset >> 3)) << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (slice_param->slice_data_offset +
                        (slice_data_bit_offset >> 3)));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (0 << 31) | /* concealment mode: 0->intra 16x16 prediction, 1->inter P Copy */
                       (0 << 14) | /* ignore BSDPrematureComplete Error handling */
                       (0 << 13) | /* FIXME: ??? */
@@ -814,12 +814,12 @@ ironlake_avc_bsd_object(VADriverContextP ctx,
                       (0 << 10) | /* ignore Entropy Error handling */
                       (0 << 8)  | /* ignore MB Header Error handling */
                       (slice_type << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (num_ref_idx_l1 << 24) |
                       (num_ref_idx_l0 << 16) |
                       (slice_param->chroma_log2_weight_denom << 8) |
                       (slice_param->luma_log2_weight_denom << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (weighted_pred_idc << 30) |
                       (slice_param->direct_spatial_mv_pred_flag << 29) |
                       (slice_param->disable_deblocking_filter_idc << 27) |
@@ -827,44 +827,44 @@ ironlake_avc_bsd_object(VADriverContextP ctx,
                       ((pic_param->pic_init_qp_minus26 + 26 + slice_param->slice_qp_delta) << 16) |
                       ((slice_param->slice_beta_offset_div2 & 0xf) << 8) |
                       ((slice_param->slice_alpha_c0_offset_div2 & 0xf) << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (slice_ver_pos << 24) |
                       (slice_hor_pos << 16) | 
                       (first_mb_in_slice << 0));
-        OUT_BCS_BATCH(ctx, 
+        OUT_BCS_BATCH(batch, 
                       (0 << 7) | /* FIXME: ??? */
                       ((0x7 - (slice_data_bit_offset & 0x7)) << 0));
-        OUT_BCS_BATCH(ctx, counter_value);
+        OUT_BCS_BATCH(batch, counter_value);
         
         /* FIXME: dw9-dw11 */
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, i965_h264_context->weight128_luma_l0);
-        OUT_BCS_BATCH(ctx, i965_h264_context->weight128_luma_l1);
-        OUT_BCS_BATCH(ctx, i965_h264_context->weight128_chroma_l0);
-        OUT_BCS_BATCH(ctx, i965_h264_context->weight128_chroma_l1);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, i965_h264_context->weight128_luma_l0);
+        OUT_BCS_BATCH(batch, i965_h264_context->weight128_luma_l1);
+        OUT_BCS_BATCH(batch, i965_h264_context->weight128_chroma_l0);
+        OUT_BCS_BATCH(batch, i965_h264_context->weight128_chroma_l1);
 
-        ADVANCE_BCS_BATCH(ctx); 
+        ADVANCE_BCS_BATCH(batch); 
     } else {
-        BEGIN_BCS_BATCH(ctx, 16);
-        OUT_BCS_BATCH(ctx, CMD_AVC_BSD_OBJECT | (16 - 2));
-        OUT_BCS_BATCH(ctx, 0); /* indirect data length for phantom slice is 0 */
-        OUT_BCS_BATCH(ctx, 0); /* indirect data start address for phantom slice is 0 */
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, width_in_mbs * height_in_mbs / (1 + !!pic_param->pic_fields.bits.field_pic_flag));
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        OUT_BCS_BATCH(ctx, 0);
-        ADVANCE_BCS_BATCH(ctx);
+        BEGIN_BCS_BATCH(batch, 16);
+        OUT_BCS_BATCH(batch, CMD_AVC_BSD_OBJECT | (16 - 2));
+        OUT_BCS_BATCH(batch, 0); /* indirect data length for phantom slice is 0 */
+        OUT_BCS_BATCH(batch, 0); /* indirect data start address for phantom slice is 0 */
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, width_in_mbs * height_in_mbs / (1 + !!pic_param->pic_fields.bits.field_pic_flag));
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        OUT_BCS_BATCH(batch, 0);
+        ADVANCE_BCS_BATCH(batch);
     }
 }
 
@@ -873,31 +873,32 @@ i965_avc_bsd_object(VADriverContextP ctx,
                     struct decode_state *decode_state,
                     VAPictureParameterBufferH264 *pic_param,
                     VASliceParameterBufferH264 *slice_param,
-                    int slice_index)
+                    int slice_index,
+                    struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
 
     if (IS_IRONLAKE(i965->intel.device_id))
-        ironlake_avc_bsd_object(ctx, decode_state, pic_param, slice_param, slice_index);
+        ironlake_avc_bsd_object(ctx, decode_state, pic_param, slice_param, slice_index, i965_h264_context);
     else
-        g4x_avc_bsd_object(ctx, decode_state, pic_param, slice_param, slice_index);
+        g4x_avc_bsd_object(ctx, decode_state, pic_param, slice_param, slice_index, i965_h264_context);
 }
 
 static void
 i965_avc_bsd_phantom_slice(VADriverContextP ctx, 
                            struct decode_state *decode_state,
-                           VAPictureParameterBufferH264 *pic_param)
+                           VAPictureParameterBufferH264 *pic_param,
+                           struct i965_h264_context *i965_h264_context)
 {
-    i965_avc_bsd_object(ctx, decode_state, pic_param, NULL, 0);
+    i965_avc_bsd_object(ctx, decode_state, pic_param, NULL, 0, i965_h264_context);
 }
 
 static void
 i965_avc_bsd_frame_store_index(VADriverContextP ctx,
-                               VAPictureParameterBufferH264 *pic_param)
+                               VAPictureParameterBufferH264 *pic_param,
+                               struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     int i, j;
 
     assert(ARRAY_ELEMS(i965_h264_context->fsid_list) == ARRAY_ELEMS(pic_param->ReferenceFrames));
@@ -957,17 +958,9 @@ i965_avc_bsd_frame_store_index(VADriverContextP ctx,
         if (!found) {
             int frame_idx;
             struct object_surface *obj_surface = SURFACE(ref_pic->picture_id);
+            assert(obj_surface);
+            i965_check_alloc_surface_bo(ctx, obj_surface, 0, VA_FOURCC('N','V','1','2'));
             
-            if (obj_surface->bo == NULL) {
-                /* Some broken sources such as conformance case FM2_SVA_C
-                 * will get here !!!. Allocating a BO for it to avoid SEGMENT FAULT
-                 */
-                obj_surface->bo = dri_bo_alloc(i965->intel.bufmgr,
-                                               "vaapi surface",
-                                               obj_surface->size,
-                                               0x1000);
-            }
-
             for (frame_idx = 0; frame_idx < ARRAY_ELEMS(i965_h264_context->fsid_list); frame_idx++) {
                 for (j = 0; j < ARRAY_ELEMS(i965_h264_context->fsid_list); j++) {
                     if (i965_h264_context->fsid_list[j].surface_id == VA_INVALID_ID)
@@ -1015,18 +1008,17 @@ i965_avc_bsd_frame_store_index(VADriverContextP ctx,
 }
 
 void 
-i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state, void *h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
+    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)h264_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     VAPictureParameterBufferH264 *pic_param;
     VASliceParameterBufferH264 *slice_param;
     int i, j;
 
     assert(decode_state->pic_param && decode_state->pic_param->buffer);
     pic_param = (VAPictureParameterBufferH264 *)decode_state->pic_param->buffer;
-    i965_avc_bsd_frame_store_index(ctx, pic_param);
+    i965_avc_bsd_frame_store_index(ctx, pic_param, i965_h264_context);
 
     i965_h264_context->enable_avc_ildb = 0;
     i965_h264_context->picture.i_flag = 1;
@@ -1035,7 +1027,6 @@ i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state)
         assert(decode_state->slice_params && decode_state->slice_params[j]->buffer);
         slice_param = (VASliceParameterBufferH264 *)decode_state->slice_params[j]->buffer;
 
-        assert(decode_state->slice_params[j]->num_elements == 1);  /* FIXME */
         for (i = 0; i < decode_state->slice_params[j]->num_elements; i++) {
             assert(slice_param->slice_data_flag == VA_SLICE_DATA_FLAG_ALL);
             assert((slice_param->slice_type == SLICE_TYPE_I) ||
@@ -1053,18 +1044,17 @@ i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state)
         }
     }
 
-    intel_batchbuffer_start_atomic_bcs(ctx, 0x1000);
+    intel_batchbuffer_start_atomic_bcs(batch, 0x1000);
 
-    i965_avc_bsd_img_state(ctx, decode_state);
-    i965_avc_bsd_qm_state(ctx, decode_state);
+    i965_avc_bsd_img_state(ctx, decode_state, i965_h264_context);
+    i965_avc_bsd_qm_state(ctx, decode_state, i965_h264_context);
 
     for (j = 0; j < decode_state->num_slice_params; j++) {
         assert(decode_state->slice_params && decode_state->slice_params[j]->buffer);
         slice_param = (VASliceParameterBufferH264 *)decode_state->slice_params[j]->buffer;
 
-        i965_bsd_ind_obj_base_address(ctx, decode_state, j);
+        i965_bsd_ind_obj_base_address(ctx, decode_state, j, i965_h264_context);
 
-        assert(decode_state->slice_params[j]->num_elements == 1);  /* FIXME */
         for (i = 0; i < decode_state->slice_params[j]->num_elements; i++) {
             assert(slice_param->slice_data_flag == VA_SLICE_DATA_FLAG_ALL);
             assert((slice_param->slice_type == SLICE_TYPE_I) ||
@@ -1078,25 +1068,24 @@ i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state)
                  slice_param->slice_type != SLICE_TYPE_SI))
                 i965_h264_context->picture.i_flag = 0;
 
-            i965_avc_bsd_slice_state(ctx, pic_param, slice_param);
-            i965_avc_bsd_buf_base_state(ctx, pic_param, slice_param);
-            i965_avc_bsd_object(ctx, decode_state, pic_param, slice_param, j);
+            i965_avc_bsd_slice_state(ctx, pic_param, slice_param, i965_h264_context);
+            i965_avc_bsd_buf_base_state(ctx, pic_param, slice_param, i965_h264_context);
+            i965_avc_bsd_object(ctx, decode_state, pic_param, slice_param, j, i965_h264_context);
             slice_param++;
         }
     }
 
-    i965_avc_bsd_phantom_slice(ctx, decode_state, pic_param);
-    intel_batchbuffer_emit_mi_flush_bcs(ctx);
-    intel_batchbuffer_end_atomic_bcs(ctx);
-    intel_batchbuffer_flush_bcs(ctx);
+    i965_avc_bsd_phantom_slice(ctx, decode_state, pic_param, i965_h264_context);
+    intel_batchbuffer_emit_mi_flush(batch);
+    intel_batchbuffer_end_atomic(batch);
+    intel_batchbuffer_flush(batch);
 }
 
 void 
-i965_avc_bsd_decode_init(VADriverContextP ctx)
+i965_avc_bsd_decode_init(VADriverContextP ctx, void *h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
+    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)h264_context;
     struct i965_avc_bsd_context *i965_avc_bsd_context;
     dri_bo *bo;
 
@@ -1118,10 +1107,6 @@ i965_avc_bsd_decode_init(VADriverContextP ctx)
                       64);
     assert(bo);
     i965_avc_bsd_context->mpr_row_store.bo = bo;
-
-    if (!i965_avc_bsd_context->init) {
-        i965_avc_bsd_context->init = 1;
-    }
 }
 
 Bool 
