@@ -123,17 +123,15 @@ struct avc_ildb_root_input
     unsigned int pad3;
 };
 
-extern struct media_kernel *h264_avc_kernels;
-
 #define NUM_AVC_ILDB_INTERFACES ARRAY_ELEMS(avc_ildb_kernel_offset_gen4)
 static unsigned long *avc_ildb_kernel_offset = NULL;
 
 static void
-i965_avc_ildb_surface_state(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_ildb_surface_state(VADriverContextP ctx,
+                            struct decode_state *decode_state,
+                            struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
     struct i965_surface_state *ss;
     struct object_surface *obj_surface;
@@ -237,11 +235,8 @@ i965_avc_ildb_surface_state(VADriverContextP ctx, struct decode_state *decode_st
 }
 
 static void
-i965_avc_ildb_binding_table(VADriverContextP ctx)
+i965_avc_ildb_binding_table(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
     unsigned int *binding_table;
     dri_bo *bo = avc_ildb_context->binding_table.bo;
@@ -265,11 +260,8 @@ i965_avc_ildb_binding_table(VADriverContextP ctx)
 }
 
 static void
-i965_avc_ildb_interface_descriptor_table(VADriverContextP ctx)
+i965_avc_ildb_interface_descriptor_table(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
     struct i965_interface_descriptor *desc;
     dri_bo *bo;
@@ -284,7 +276,7 @@ i965_avc_ildb_interface_descriptor_table(VADriverContextP ctx)
         int kernel_offset = avc_ildb_kernel_offset[i];
         memset(desc, 0, sizeof(*desc));
         desc->desc0.grf_reg_blocks = 7; 
-        desc->desc0.kernel_start_pointer = (h264_avc_kernels[H264_AVC_COMBINED].bo->offset + kernel_offset) >> 6; /* reloc */
+        desc->desc0.kernel_start_pointer = (i965_h264_context->avc_kernels[H264_AVC_COMBINED].bo->offset + kernel_offset) >> 6; /* reloc */
         desc->desc1.const_urb_entry_read_offset = 0;
         desc->desc1.const_urb_entry_read_len = ((i == AVC_ILDB_ROOT_Y_ILDB_FRAME ||
                                                  i == AVC_ILDB_ROOT_Y_ILDB_FIELD ||
@@ -297,7 +289,7 @@ i965_avc_ildb_interface_descriptor_table(VADriverContextP ctx)
                           I915_GEM_DOMAIN_INSTRUCTION, 0,
                           desc->desc0.grf_reg_blocks + kernel_offset,
                           i * sizeof(*desc) + offsetof(struct i965_interface_descriptor, desc0),
-                          h264_avc_kernels[H264_AVC_COMBINED].bo);
+                          i965_h264_context->avc_kernels[H264_AVC_COMBINED].bo);
 
         dri_bo_emit_reloc(bo,
                           I915_GEM_DOMAIN_INSTRUCTION, 0,
@@ -311,11 +303,8 @@ i965_avc_ildb_interface_descriptor_table(VADriverContextP ctx)
 }
 
 static void
-i965_avc_ildb_vfe_state(VADriverContextP ctx)
+i965_avc_ildb_vfe_state(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
     struct i965_vfe_state *vfe_state;
     dri_bo *bo;
@@ -341,11 +330,11 @@ i965_avc_ildb_vfe_state(VADriverContextP ctx)
 }
 
 static void
-i965_avc_ildb_upload_constants(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_ildb_upload_constants(VADriverContextP ctx,
+                               struct decode_state *decode_state,
+                               struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
     VAPictureParameterBufferH264 *pic_param;
     struct avc_ildb_root_input *root_input;
@@ -395,189 +384,183 @@ i965_avc_ildb_upload_constants(VADriverContextP ctx, struct decode_state *decode
 }
 
 static void
-i965_avc_ildb_states_setup(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_ildb_states_setup(VADriverContextP ctx,
+                           struct decode_state *decode_state,
+                           struct i965_h264_context *i965_h264_context)
 {
-    i965_avc_ildb_surface_state(ctx, decode_state);
-    i965_avc_ildb_binding_table(ctx);
-    i965_avc_ildb_interface_descriptor_table(ctx);
-    i965_avc_ildb_vfe_state(ctx);
-    i965_avc_ildb_upload_constants(ctx, decode_state);
+    i965_avc_ildb_surface_state(ctx, decode_state, i965_h264_context);
+    i965_avc_ildb_binding_table(ctx, i965_h264_context);
+    i965_avc_ildb_interface_descriptor_table(ctx, i965_h264_context);
+    i965_avc_ildb_vfe_state(ctx, i965_h264_context);
+    i965_avc_ildb_upload_constants(ctx, decode_state, i965_h264_context);
 }
 
 static void
-i965_avc_ildb_pipeline_select(VADriverContextP ctx)
+i965_avc_ildb_pipeline_select(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    BEGIN_BATCH(ctx, 1);
-    OUT_BATCH(ctx, CMD_PIPELINE_SELECT | PIPELINE_SELECT_MEDIA);
-    ADVANCE_BATCH(ctx);
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
+
+    BEGIN_BATCH(batch, 1);
+    OUT_BATCH(batch, CMD_PIPELINE_SELECT | PIPELINE_SELECT_MEDIA);
+    ADVANCE_BATCH(batch);
 }
 
 static void
-i965_avc_ildb_urb_layout(VADriverContextP ctx)
+i965_avc_ildb_urb_layout(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
-
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
     unsigned int vfe_fence, cs_fence;
 
     vfe_fence = avc_ildb_context->urb.cs_start;
     cs_fence = URB_SIZE((&i965->intel));
 
-    BEGIN_BATCH(ctx, 3);
-    OUT_BATCH(ctx, CMD_URB_FENCE | UF0_VFE_REALLOC | UF0_CS_REALLOC | 1);
-    OUT_BATCH(ctx, 0);
-    OUT_BATCH(ctx, 
+    BEGIN_BATCH(batch, 3);
+    OUT_BATCH(batch, CMD_URB_FENCE | UF0_VFE_REALLOC | UF0_CS_REALLOC | 1);
+    OUT_BATCH(batch, 0);
+    OUT_BATCH(batch, 
               (vfe_fence << UF2_VFE_FENCE_SHIFT) |      /* VFE_SIZE */
               (cs_fence << UF2_CS_FENCE_SHIFT));        /* CS_SIZE */
-    ADVANCE_BATCH(ctx);
+    ADVANCE_BATCH(batch);
 }
 
 static void
-i965_avc_ildb_state_base_address(VADriverContextP ctx)
+i965_avc_ildb_state_base_address(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx); 
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
 
     if (IS_IRONLAKE(i965->intel.device_id)) {
-        BEGIN_BATCH(ctx, 8);
-        OUT_BATCH(ctx, CMD_STATE_BASE_ADDRESS | 6);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        ADVANCE_BATCH(ctx);
+        BEGIN_BATCH(batch, 8);
+        OUT_BATCH(batch, CMD_STATE_BASE_ADDRESS | 6);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        ADVANCE_BATCH(batch);
     } else {
-        BEGIN_BATCH(ctx, 6);
-        OUT_BATCH(ctx, CMD_STATE_BASE_ADDRESS | 4);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
-        ADVANCE_BATCH(ctx);
+        BEGIN_BATCH(batch, 6);
+        OUT_BATCH(batch, CMD_STATE_BASE_ADDRESS | 4);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        OUT_BATCH(batch, 0 | BASE_ADDRESS_MODIFY);
+        ADVANCE_BATCH(batch);
     }
 }
 
 static void
-i965_avc_ildb_state_pointers(VADriverContextP ctx)
+i965_avc_ildb_state_pointers(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
 
-    BEGIN_BATCH(ctx, 3);
-    OUT_BATCH(ctx, CMD_MEDIA_STATE_POINTERS | 1);
-    OUT_BATCH(ctx, 0);
-    OUT_RELOC(ctx, avc_ildb_context->vfe_state.bo, I915_GEM_DOMAIN_INSTRUCTION, 0, 0);
-    ADVANCE_BATCH(ctx);
+    BEGIN_BATCH(batch, 3);
+    OUT_BATCH(batch, CMD_MEDIA_STATE_POINTERS | 1);
+    OUT_BATCH(batch, 0);
+    OUT_RELOC(batch, avc_ildb_context->vfe_state.bo, I915_GEM_DOMAIN_INSTRUCTION, 0, 0);
+    ADVANCE_BATCH(batch);
 }
 
 static void 
-i965_avc_ildb_cs_urb_layout(VADriverContextP ctx)
+i965_avc_ildb_cs_urb_layout(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
 
-    BEGIN_BATCH(ctx, 2);
-    OUT_BATCH(ctx, CMD_CS_URB_STATE | 0);
-    OUT_BATCH(ctx,
+    BEGIN_BATCH(batch, 2);
+    OUT_BATCH(batch, CMD_CS_URB_STATE | 0);
+    OUT_BATCH(batch,
               ((avc_ildb_context->urb.size_cs_entry - 1) << 4) |     /* URB Entry Allocation Size */
               (avc_ildb_context->urb.num_cs_entries << 0));          /* Number of URB Entries */
-    ADVANCE_BATCH(ctx);
+    ADVANCE_BATCH(batch);
 }
 
 static void
-i965_avc_ildb_constant_buffer(VADriverContextP ctx)
+i965_avc_ildb_constant_buffer(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
 
-    BEGIN_BATCH(ctx, 2);
-    OUT_BATCH(ctx, CMD_CONSTANT_BUFFER | (1 << 8) | (2 - 2));
-    OUT_RELOC(ctx, avc_ildb_context->curbe.bo,
+    BEGIN_BATCH(batch, 2);
+    OUT_BATCH(batch, CMD_CONSTANT_BUFFER | (1 << 8) | (2 - 2));
+    OUT_RELOC(batch, avc_ildb_context->curbe.bo,
               I915_GEM_DOMAIN_INSTRUCTION, 0,
               avc_ildb_context->urb.size_cs_entry - 1);
-    ADVANCE_BATCH(ctx);    
+    ADVANCE_BATCH(batch);    
 }
 
 static void
-i965_avc_ildb_objects(VADriverContextP ctx)
+i965_avc_ildb_objects(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
 
-    BEGIN_BATCH(ctx, 6);
-    OUT_BATCH(ctx, CMD_MEDIA_OBJECT | 4);
+    BEGIN_BATCH(batch, 6);
+    OUT_BATCH(batch, CMD_MEDIA_OBJECT | 4);
 
     switch (avc_ildb_context->picture_type) {
     case PICTURE_FRAME:
-        OUT_BATCH(ctx, AVC_ILDB_ROOT_Y_ILDB_FRAME);
+        OUT_BATCH(batch, AVC_ILDB_ROOT_Y_ILDB_FRAME);
         break;
 
     case PICTURE_FIELD:
-        OUT_BATCH(ctx, AVC_ILDB_ROOT_Y_ILDB_FIELD);
+        OUT_BATCH(batch, AVC_ILDB_ROOT_Y_ILDB_FIELD);
         break;
 
     case PICTURE_MBAFF:
-        OUT_BATCH(ctx, AVC_ILDB_ROOT_Y_ILDB_MBAFF);
+        OUT_BATCH(batch, AVC_ILDB_ROOT_Y_ILDB_MBAFF);
         break;
 
     default:
         assert(0);
-        OUT_BATCH(ctx, 0);
+        OUT_BATCH(batch, 0);
         break;
     }
 
-    OUT_BATCH(ctx, 0); /* no indirect data */
-    OUT_BATCH(ctx, 0);
-    OUT_BATCH(ctx, 0);
-    OUT_BATCH(ctx, 0);
-    ADVANCE_BATCH(ctx);
+    OUT_BATCH(batch, 0); /* no indirect data */
+    OUT_BATCH(batch, 0);
+    OUT_BATCH(batch, 0);
+    OUT_BATCH(batch, 0);
+    ADVANCE_BATCH(batch);
 }
 
 static void
-i965_avc_ildb_pipeline_setup(VADriverContextP ctx)
+i965_avc_ildb_pipeline_setup(VADriverContextP ctx, struct i965_h264_context *i965_h264_context)
 {
-    intel_batchbuffer_emit_mi_flush(ctx);
-    i965_avc_ildb_pipeline_select(ctx);
-    i965_avc_ildb_state_base_address(ctx);
-    i965_avc_ildb_state_pointers(ctx);
-    i965_avc_ildb_urb_layout(ctx);
-    i965_avc_ildb_cs_urb_layout(ctx);
-    i965_avc_ildb_constant_buffer(ctx);
-    i965_avc_ildb_objects(ctx);
+    struct intel_batchbuffer *batch = i965_h264_context->batch;
+
+    intel_batchbuffer_emit_mi_flush(batch);
+    i965_avc_ildb_pipeline_select(ctx, i965_h264_context);
+    i965_avc_ildb_state_base_address(ctx, i965_h264_context);
+    i965_avc_ildb_state_pointers(ctx, i965_h264_context);
+    i965_avc_ildb_urb_layout(ctx, i965_h264_context);
+    i965_avc_ildb_cs_urb_layout(ctx, i965_h264_context);
+    i965_avc_ildb_constant_buffer(ctx, i965_h264_context);
+    i965_avc_ildb_objects(ctx, i965_h264_context);
 }
 
 void
-i965_avc_ildb(VADriverContextP ctx, struct decode_state *decode_state)
+i965_avc_ildb(VADriverContextP ctx, struct decode_state *decode_state, void *h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
+    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)h264_context;
 
     if (i965_h264_context->enable_avc_ildb) {
-        i965_avc_ildb_states_setup(ctx, decode_state);
-        i965_avc_ildb_pipeline_setup(ctx);
+        i965_avc_ildb_states_setup(ctx, decode_state, i965_h264_context);
+        i965_avc_ildb_pipeline_setup(ctx, i965_h264_context);
     }
 }
 
 void
-i965_avc_ildb_decode_init(VADriverContextP ctx)
+i965_avc_ildb_decode_init(VADriverContextP ctx, void *h264_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_media_state *media_state = &i965->media_state;
-    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)media_state->private_context;
+    struct i965_h264_context *i965_h264_context = (struct i965_h264_context *)h264_context;
     struct i965_avc_ildb_context *avc_ildb_context = &i965_h264_context->avc_ildb_context;;
     dri_bo *bo;
     int i;
