@@ -1451,9 +1451,9 @@ VAStatus vaQuerySurfaceStatus (
 
 typedef enum
 {
-    VA_DECODE_SLICE_MISSING            = 0,
-    VA_DECODE_MB_ERROR                 = 1,
-} VA_DECODE_ERROR_TYPE;
+    VADecodeSliceMissing            = 0,
+    VADecodeMBError                 = 1,
+} VADecodeErrorType;
 
 /*
  * Client calls vaQuerySurfaceError with VA_STATUS_ERROR_DECODING_ERROR, server side returns
@@ -1464,7 +1464,7 @@ typedef struct _VASurfaceDecodeMBErrors
     int status; /* 1 if hardware has returned detailed info below, -1 means this record is invalid */
     unsigned int start_mb; /* start mb address with errors */
     unsigned int end_mb;  /* end mb address with errors */
-    VA_DECODE_ERROR_TYPE decode_error_type;
+    VADecodeErrorType decode_error_type;
 } VASurfaceDecodeMBErrors;
 
 /*
@@ -1497,6 +1497,7 @@ VAStatus vaQuerySurfaceError(
 #define VA_FOURCC_NV12		0x3231564E
 #define VA_FOURCC_AI44		0x34344149
 #define VA_FOURCC_RGBA		0x41424752
+#define VA_FOURCC_BGRA		0x41524742
 #define VA_FOURCC_UYVY          0x59565955
 #define VA_FOURCC_YUY2          0x32595559
 #define VA_FOURCC_AYUV          0x56555941
@@ -1845,7 +1846,7 @@ typedef enum
 #define VA_ROTATION_NONE        0x00000000
 #define VA_ROTATION_90          0x00000001
 #define VA_ROTATION_180         0x00000002
-#define VA_ROTATION_270         0x00000004
+#define VA_ROTATION_270         0x00000003
 
 /* attribute value for VADisplayAttribOutOfLoopDeblock */
 #define VA_OOL_DEBLOCKING_FALSE 0x00000000
